#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Author : 陈坤泽
# @Email  : 877362867@qq.com
# @Date   : 2021/06/06 22:58

from pyxllib.xlcv import *

from pyxllib.ai.torch import *
from pyxllib.ai.torch_app import *

from pyxllib.data.icdar import IcdarEval
from pyxllib.data.labelme import *
from pyxllib.data.coco import *
from pyxllib.data.specialist import *
